"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

const preventCollapse = .05;

function capsize(options) {
  if ("leading" in options && "lineGap" in options) throw new Error("Only a single line height style can be provided. Please pass either `lineGap` OR `leading`.");
  if ("capHeight" in options && "fontSize" in options) throw new Error("Please pass either `capHeight` OR `fontSize`, not both.");
  const {fontMetrics: fontMetrics} = options, capHeightScale = fontMetrics.capHeight / fontMetrics.unitsPerEm;
  let specifiedFontSize, specifiedCapHeight, specifiedLineHeight;
  if ("capHeight" in options) specifiedFontSize = options.capHeight / capHeightScale, 
  specifiedCapHeight = options.capHeight; else {
    if (!("fontSize" in options)) throw new Error("Please pass either `capHeight` OR `fontSize`.");
    specifiedFontSize = options.fontSize, specifiedCapHeight = options.fontSize * capHeightScale;
  }
  return "lineGap" in options ? specifiedLineHeight = specifiedCapHeight + options.lineGap : "leading" in options && (specifiedLineHeight = options.leading), 
  createCss({
    lineHeight: specifiedLineHeight,
    fontSize: specifiedFontSize,
    fontMetrics: fontMetrics
  });
}

function createCss({lineHeight: lineHeight, fontSize: fontSize, fontMetrics: fontMetrics}) {
  const toScale = value => value / fontSize, absoluteDescent = Math.abs(fontMetrics.descent), capHeightScale = fontMetrics.capHeight / fontMetrics.unitsPerEm, descentScale = absoluteDescent / fontMetrics.unitsPerEm, ascentScale = fontMetrics.ascent / fontMetrics.unitsPerEm, lineGapScale = fontMetrics.lineGap / fontMetrics.unitsPerEm, lineHeightScale = (fontMetrics.ascent + fontMetrics.lineGap + absoluteDescent) / fontMetrics.unitsPerEm, specifiedLineHeightOffset = lineHeight ? (lineHeightScale * fontSize - lineHeight) / 2 : 0, leadingTrim = value => value - toScale(specifiedLineHeightOffset) + toScale(.05);
  return {
    fontSize: fontSize + "px",
    lineHeight: lineHeight ? lineHeight + "px" : "normal",
    padding: "0.05px 0",
    "::before": {
      content: "''",
      marginTop: -1 * leadingTrim(ascentScale - capHeightScale + lineGapScale / 2) + "em",
      display: "block",
      height: 0
    },
    "::after": {
      content: "''",
      marginBottom: -1 * leadingTrim(descentScale + lineGapScale / 2) + "em",
      display: "block",
      height: 0
    }
  };
}

exports.default = capsize;
